#include <ESP8266WiFi.h>
#include <ESP8266WiFiAP.h>
#include <ESP8266WiFiGeneric.h>
#include <ESP8266WiFiMulti.h>
#include <ESP8266WiFiScan.h>
#include <ESP8266WiFiSTA.h>
#include <ESP8266WiFiType.h>
#include <WiFiClient.h>
#include <WiFiClientSecure.h>
#include <WiFiServer.h>
#include <WiFiServerSecure.h>
#include <Arduino.h>
#include <EasyNTPClient.h>
#include <WiFiUdp.h>
#include <TimeLib.h>
#include <Wire.h>  

/**************************************************/
// ntp init 
WiFiUDP udp;
EasyNTPClient ntpClient(udp, "192.168.1.1", ((2*60*60))); // IST = GMT + 1
//EasyNTPClient ntpClient(udp, "pool.ntp.org", 0);
//EasyNTPClient ntpClient(udp, "192.168.1.1");


/**************************************************/
// Wifi settings
const char* ssid     = "NODE_GROUND";
const char* password = "IAmAMeshNetworkPassword";

/**************************************************/
// init display 
#include "SSD1306Wire.h" 
#include "OLEDDisplayUi.h"
#include "images.h"
SSD1306Wire  display(0x3c, D5, D6);
OLEDDisplayUi ui ( &display );
int screenW = 128;
int screenH = 64;
int clockCenterX = screenW / 2;
int clockCenterY = ((screenH - 16) / 2) + 16; // top yellow part is 16 px height
int clockRadius = 23;

void analogClockFrame(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y);
void networkDataFrame(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y);

// This array keeps function pointers to all frames
// frames are the single views that slide in
FrameCallback frames[] = { analogClockFrame, networkDataFrame};

// how many frames are there?
int frameCount = 2;

void clockOverlay(OLEDDisplay *display, OLEDDisplayUiState* state) {

}
// Overlays are statically drawn on top of a frame eg. a clock
OverlayCallback overlays[] = { clockOverlay };
int overlaysCount = 1;
/**************************************************/

String twoDigits(int digits){
  if(digits < 10) {
    String i = '0'+String(digits);
    return i;
  }
  else {
    return String(digits);
  }
}


void initDisplay(){
  // The ESP is capable of rendering 60fps in 80Mhz mode
  // but that won't give you much time for anything else
  // run it in 160Mhz mode or just set it to 30 fps
  ui.setTargetFPS(30);

  // You can change this to
  // TOP, LEFT, BOTTOM, RIGHT
  ui.setIndicatorPosition(TOP);
  
  ui.setActiveSymbol(activeSymbol);
  ui.setInactiveSymbol(inactiveSymbol);
  
  // Add frames
  ui.setFrames(frames, frameCount);

  // Add overlays
  ui.setOverlays(overlays, overlaysCount);

  // Initialising the UI will init the display too.
  ui.init();

  display.flipScreenVertically();
}
void analogClockFrame(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) {
  //  ui.disableIndicator();

  // Draw the clock face
  //  display->drawCircle(clockCenterX + x, clockCenterY + y, clockRadius);
  display->drawCircle(clockCenterX + x, clockCenterY + y, 2);
  //
  //hour ticks
  for ( int z = 0; z < 360; z = z + 30 ) {
    //Begin at 0° and stop at 360°
    float angle = z ;
    angle = ( angle / 57.29577951 ) ; //Convert degrees to radians
    int x2 = ( clockCenterX + ( sin(angle) * clockRadius ) );
    int y2 = ( clockCenterY - ( cos(angle) * clockRadius ) );
    int x3 = ( clockCenterX + ( sin(angle) * ( clockRadius - ( clockRadius / 8 ) ) ) );
    int y3 = ( clockCenterY - ( cos(angle) * ( clockRadius - ( clockRadius / 8 ) ) ) );
    display->drawLine( x2 + x , y2 + y , x3 + x , y3 + y);
  }

  // display second hand
  float angle = second() * 6 ;
  angle = ( angle / 57.29577951 ) ; //Convert degrees to radians
  int x3 = ( clockCenterX + ( sin(angle) * ( clockRadius - ( clockRadius / 5 ) ) ) );
  int y3 = ( clockCenterY - ( cos(angle) * ( clockRadius - ( clockRadius / 5 ) ) ) );
  display->drawLine( clockCenterX + x , clockCenterY + y , x3 + x , y3 + y);
  //
  // display minute hand
  angle = minute() * 6 ;
  angle = ( angle / 57.29577951 ) ; //Convert degrees to radians
  x3 = ( clockCenterX + ( sin(angle) * ( clockRadius - ( clockRadius / 4 ) ) ) );
  y3 = ( clockCenterY - ( cos(angle) * ( clockRadius - ( clockRadius / 4 ) ) ) );
  display->drawLine( clockCenterX + x , clockCenterY + y , x3 + x , y3 + y);
  //
  // display hour hand
  angle = hour() * 30 + int( ( minute() / 12 ) * 6 )   ;
  angle = ( angle / 57.29577951 ) ; //Convert degrees to radians
  x3 = ( clockCenterX + ( sin(angle) * ( clockRadius - ( clockRadius / 2 ) ) ) );
  y3 = ( clockCenterY - ( cos(angle) * ( clockRadius - ( clockRadius / 2 ) ) ) );
  display->drawLine( clockCenterX + x , clockCenterY + y , x3 + x , y3 + y);
}

void networkDataFrame(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) {
  //String timenow = String(hour()) + ":" + twoDigits(minute()) + ":" + twoDigits(second());
  
  display->setTextAlignment(TEXT_ALIGN_CENTER);
  display->setFont(ArialMT_Plain_10);
  display->drawString(clockCenterX + x  , clockCenterY + y -20, "MAC: "+WiFi.macAddress() );
  display->drawString(clockCenterX + x  , clockCenterY + y -10, "IP : "+WiFi.localIP().toString() );
  //display->drawString(clockCenterX + x , clockCenterY + y, WiFi.localIP().toString() );
  display->drawString(clockCenterX + x , clockCenterY + y+10, twoDigits(hour())+":"+twoDigits(minute())+":"+twoDigits(+second()) );
  
}

void setNTPTime(){
  setTime(ntpClient.getUnixTime());
     unsigned long secsSinceStart = ntpClient.getUnixTime();
      // Unix time starts on Jan 1 1970. In seconds, that's 2208988800:
      const unsigned long seventyYears = 2208988800UL;
      // subtract seventy years:
      unsigned long epoch = secsSinceStart - seventyYears * SECS_PER_HOUR;
      //setTime(epoch);
}

void setup() {
  Serial.begin(115200);
  Serial.println();


  // We start by connecting to a WiFi network,
  WiFi.begin(ssid, password);
  Serial.println("Connecting to ");
  Serial.println(String(ssid));
  int timeout =0;
  while (WiFi.status() != WL_CONNECTED && timeout < 200){
    Serial.print(".");
    delay(100);
    timeout ++;
  }
   
  if (WiFi.status() != WL_CONNECTED) {
    Serial.println("Failed");
    delay(500);
    ESP.restart();
  }
  Serial.println("Success");
  Serial.println("MAC: " + WiFi.macAddress());
  Serial.println("");
  Serial.println("WiFi connected ");
  Serial.println(WiFi.localIP().toString());

  Serial.print("connecting to ");
  
  setNTPTime();

  initDisplay();

}


void loop() {
  int remainingTimeBudget = ui.update();

  if (remainingTimeBudget > 0) {
     setNTPTime();
    delay(remainingTimeBudget);
  }


}
