#ifndef NTP_H
#define NTP_H


#include <Time.h>
#include <TimeLib.h>
/*
#if defined ( ESP8266 )
  #include <pgmspace.h>
#else
  #include <avr/pgmspace.h>
#endif
*/

#ifdef PROGMEM
  #undef PROGMEM
  #define PROGMEM __attribute__((section(".progmem.data")))
#endif

#define NTP_DEBUG

#ifndef NTP_SERVER
  #define NTP_SERVER "192.168.1.1"
#endif // NTP_SERVER

#ifndef UDP_PORT
  #define UDP_PORT 2390
#endif //UDP_PORT

#ifndef TIME_ZONE
  #define TIME_ZONE 0 // multiply if more than 0* SECS_PER_HOUR
#endif //TIME_ZONE

// NTP time is in the first 48 bytes of message
const int NTP_PACKET_SIZE = 48;

//buffer to hold incoming & outgoing packet
byte packetBuffer[NTP_PACKET_SIZE];

WiFiUDP Udp;


// send an NTP request to the time server at the given address
void sendNTPpacket()
{
  // set all bytes in the buffer to 0
  memset(packetBuffer, 0, NTP_PACKET_SIZE);
  // Initialize values needed to form NTP request
  // (see URL above for details on the packets)
  packetBuffer[0] = 0b11100011;   // LI, Version, Mode
  packetBuffer[1] = 0;     // Stratum, or type of clock
  packetBuffer[2] = 6;     // Polling Interval
  packetBuffer[3] = 0xEC;  // Peer Clock Precision
  // 8 bytes of zero for Root Delay & Root Dispersion
  packetBuffer[12] = 49;
  packetBuffer[13] = 0x4E;
  packetBuffer[14] = 49;
  packetBuffer[15] = 52;
  // all NTP fields have been given values, now
  // you can send a packet requesting a timestamp:
  Udp.beginPacket(NTP_SERVER, 123); //NTP requests are to port 123
  Udp.write(packetBuffer, NTP_PACKET_SIZE);
  Udp.endPacket();
}

time_t getNtpTime()
{

  while (Udp.parsePacket() > 0) ; // discard any previously received packets
#ifdef NTP_DEBUG
  Serial.println(PSTR("Transmit NTP Request"));
#endif

  Serial.print(": ");
  Serial.println(NTP_SERVER);
  sendNTPpacket();
  uint32_t beginWait = millis();
  while (millis() - beginWait < 1500) {
    int size = Udp.parsePacket();
    if (size >= NTP_PACKET_SIZE) {
      #ifdef NTP_DEBUG
      Serial.println(PSTR("Receive NTP Response"));
      #endif
      Udp.read(packetBuffer, NTP_PACKET_SIZE);  // read packet into the buffer
      unsigned long secsSince1900;
      // convert four bytes starting at location 40 to a long integer
      secsSince1900 =  (unsigned long)packetBuffer[40] << 24;
      secsSince1900 |= (unsigned long)packetBuffer[41] << 16;
      secsSince1900 |= (unsigned long)packetBuffer[42] << 8;
      secsSince1900 |= (unsigned long)packetBuffer[43];
      return secsSince1900 - 2208988800UL + TIME_ZONE ;
    }
  }
  Serial.println(PSTR("No NTP Response :-("));
  return 0; // return 0 if unable to get the time
}



void initNTP() {
  Udp.begin(UDP_PORT);
  Serial.print("Local port: ");
  Serial.println(Udp.localPort());
  Serial.println("waiting for sync");
  setSyncProvider(getNtpTime);
  setSyncInterval(300);
}

#endif // NTP_H
